#!/bin/sh
#
# Filename:
# 	xbatcheck.tcl
#
# RCS id:
#	$Id: xbatcheck.tcl,v 1.2 1997/08/19 12:26:51 stefan Exp stefan $
#
# Description:
#	This tcl/tk script implements a GUI showing the current
#	battery status. The program reads the /proc/apm file 
#	and updates the GUI accordingly. the GUI is alwasy sticky,
#	meaning that on virtual screens, it stays where it is on the
#	physical screen.
#
#	The GUI consists of two images: one representing the status
#	of the battery, and one of the AC power line. Moving the mouse
#	over these images, reveils more information about the online
#	and battery status, like minutes left and percentage of battery
#	power. Clicking with any mouse buttons pops up an options window,
#	which allows you to set alarms on critical power and the check
#	interval.
#
# Requirements:
#	Kernel build with /proc and apm enabled.
#
# Copyright:
# 	Copyright (C) 1997, Stefan Sinnige. <stefan@irdeto.nl>
#
#	Permission to use, copy, modify, and distribute this software for any
# 	purpose without fee is hereby granted, provided that this entire notice
# 	is included in all copies of any software which is or includes a copy
# 	or modification of this software and in all copies of the supporting
# 	documentation for such software.
# 	THIS SOFTWARE IS BEING PROVIDED "AS IS", WITHOUT ANY EXPRESS OR IMPLIED
# 	WARRANTY OR FITNESS FOR A PARTICULAR PURPOSE.  
#
# \
/usr/bin/wish "$0" "$@" 

#========================================
#
# Option definitions. 
#
# These may be changed to cusstomize your startup appearance.
#

# Choose one of these for the placement of the widget,
# either at the left or the right side of the screen

set options(xorientation) left
#set options(xorientation) right

# Choose one of these for the placement of the widget,
# either at the top or the bottom side of the screen

#set optionsyxorientation) top
set options(yorientation) bottom

# Set the check-interval seconds (between 1 and 10). This
# is the interval which to update the GUI.
set options(interval) 5

# Set the following option to true or false to enable or
# disable battery critical power alarm
set options(alarm) true 

# Do not touch the following options !
set options(alarm_started) 0

#========================================
#
# Definition of the acline and battery bitmaps
#

image create bitmap acline_xbm -background black -data "\
#define acline_width 16\
#define acline_height 16\
static unsigned char acline_bits[] = {\
   0x00, 0x00, 0x00, 0x00, 0x00, 0x1e, 0x00, 0x7f, 0x80, 0x1f, 0xfc, 0x1f,\
   0x82, 0x1f, 0x02, 0x7f, 0x02, 0x1e, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x00,\
   0x80, 0x00, 0x7e, 0x00, 0x00, 0x00, 0x00, 0x00};"

image create bitmap battery_xbm -background black -data "\
#define battery_width 16\
#define battery_height 16\
static unsigned char battery_bits[] = {\
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0x1f, 0xfe, 0x1f, 0xfe, 0x7f,\
   0xfe, 0x5f, 0xfe, 0x5f, 0xfe, 0x7f, 0xfe, 0x1f, 0xfe, 0x1f, 0x00, 0x00,\
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};"

image create bitmap no_battery_xbm -background black -foreground red -data "\
#define crossbattery_width 16\
#define crossbattery_height 16\
static unsigned char no_battery_bits[] = {\
   0x00, 0x00, 0x00, 0x0c, 0x00, 0x06, 0xfe, 0x1f, 0x02, 0x13, 0x82, 0x71,\
   0xc2, 0x50, 0x62, 0x50, 0x32, 0x70, 0x12, 0x10, 0xfe, 0x1f, 0x0c, 0x00,\
   0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};"

#========================================
#
# balloon_setup <widget> <text>
#
# Widget ballooning. Show the <text> in a balloon for the specified
# widget whenever the focus enters the widget. Destroy the balloon
# whenever the focus leaves the widget.
#

proc balloon_setup {widget text} {
    global balloons

    set balloons($widget) $text
    bind $widget <Any-Enter> "balloon_enter $widget"
    bind $widget <Any-Leave> "balloon_leave $widget"
}

proc balloon_enter {widget} {
    global balloons options

    #
    # Construct the balloon
    #

    catch {destroy .balloon}
    toplevel .balloon
    frame .balloon.top -relief raised -borderwidth 1 -bg LightYellow2
        message .balloon.top.msg -width 300 -text $balloons($widget) \
	    -bg LightYellow2 -fg black
	pack .balloon.top.msg -side left -pady 0m
    pack .balloon.top -fill both

    wm overrideredirect .balloon 1

    #
    # Calculate the x and y position, taking into account the
    # values of the options(xorientation) and options(yorientation). 
    #

    if {$options(xorientation) == "left"} {
        set x "+[expr [winfo rootx $widget] + 10]"
    } else {
	set x "-1"
    }

    if {$options(yorientation) == "top"} {
       set y "+30"
    } else {
       set y "+[expr [winfo screenheight .] - 50]"
    }

    wm geometry .balloon $x$y
}

proc balloon_leave {widget} {
    catch {destroy .balloon}
}

#========================================
#
# apm_options
#
# The options window and change functions.
#

proc apm_options {} {
    global options

    #
    # Check if options window is already open
    #

    if [winfo exists .options] {
        return
    }

    #
    # Create the options window
    #

    toplevel .options
    wm title .options "xbatcheck options"

    set root .options.frame
    frame $root
        set w $root.xorientation
        frame $w
	    label $w.label -text "X Orientation" -width 20
            radiobutton $w.left -text Left -value left \
	        -variable  options(xorientation) -command apm_options_geometry
            radiobutton $w.right -text Right -value right \
	        -variable options(xorientation) -command apm_options_geometry
	    pack $w.label $w.left $w.right -side left
        set w $root.yorientation
        frame $w
	    label $w.label -text "Y Orientation" -width 20
            radiobutton $w.top -text Top -value top \
	        -variable  options(yorientation) -command apm_options_geometry
            radiobutton $w.bottom -text Bottom -value bottom \
	        -variable options(yorientation) -command apm_options_geometry
	    pack $w.label $w.top $w.bottom -side left
	set w $root.interval
	frame $w
	    label $w.label -text "Check interval (sec)" -width 20
	    scale $w.interval -orient horizontal -from 1 -to 10 \
	        -variable options(interval) 
	    pack $w.label $w.interval -side left 
	set w $root.alarm
	frame $w
	    label $w.label -text "Critical power alarm" -width 20
	    radiobutton $w.enable -text Enable -value true \
		-variable options(alarm) 
	    radiobutton $w.disable -text Disable -value false \
		-variable options(alarm) 
	    pack $w.label $w.enable $w.disable -side left
	set w $root.sep
	frame $w -height 3 -borderwidth 3 -relief raised
	set w $root.buttons
	frame $w
	    button $w.close -text Close -width 6 -command {
	        destroy .options
	    }
	    pack $w.close
	pack $root.xorientation -anchor w -pady 1m -padx 1m
	pack $root.yorientation -anchor w -pady 1m -padx 1m
	pack $root.interval     -anchor w -pady 1m -padx 1m
	pack $root.alarm        -anchor w -pady 1m -padx 1m
	pack $root.sep          -fill x
	pack $root.buttons      -pady 1m
    pack $root
}

proc apm_options_geometry {} {
    global options
   
    catch {destroy .balloon}

    if {$options(xorientation) == "left"} {
        set x +0
    } else {
        set x -1
    }

    if {$options(yorientation) == "top"} {
        set y +0
    } else {
        set y -1
    }

    wm geometry . $x$y
}

#========================================
#
# apm_alarm
#
# Generate an alarm message, stating that power is critical. This
# message will be repeated every minute.
#

proc apm_alarm {} {
    global options apm

    #
    # Check if still critical
    #

    if {$apm(battery_status) != 2} {
        set options(alarm_started) 0
        return
    }

    #
    # Check if alarm is enabled
    #

    if {$options(alarm) == "false"} {
        return
    }

    #
    # Check if already started
    #

    if {$options(alarm_started) == 1} {
        return
    }

    #
    # Construct a warning message
    #

    if [winfo exists .alarm] {
        raise .alarm
        focus .alarm
	return
    }

    toplevel .alarm
    set w .alarm.frame
    frame $w
        frame $w.msg
            label $w.msg.bitmap -bitmap warning -foreground red
	    label $w.msg.text -text "Critical battery power."
	    pack $w.msg.bitmap $w.msg.text -side left -pady 2m -padx 2m
	pack $w.msg
        button $w.ok -text Ok -width 6 -command "destroy .alarm"
	pack $w.ok -pady 2m -side bottom
    pack $w

    #
    # reschedule after 1 minute
    #

    after 60000 apm_alarm
}

#========================================
#
# apm_update
#
# Procedure which will update the gui to the latest statistics and 
# sets up a new timer to do the same in about 10 seconds
# 

proc apm_update {} {
    global options apm

    #
    # Open the file /proc/apm
    #

    if [catch {open /proc/apm r} fileId] {
        puts "Unable to open /proc/apm: $fileId"
        exit
    }

    #
    # Read the status line:
    #
    #    "%s %d.%d 0x%02x 0x%02x 0x%02x 0x%02x %d%% %d %s"
    #
    #        linux_driver_version
    #        (apm_bios_info.version >> 8) & 0xff
    #        apm_bios_info.version & 0xff
    #        apm_bios_info.flags
    #		bit 0 : 16 bit support
    #		bit 1 : 32 bit support
    #		bit 2 : idle slows clock
    #		bit 3 : bios disabled
    #		bit 4 : bios disengaged
    #        ac_line_status
    #		0x00 : offline
    #		0x01 : online
    #		0x02 : backup power (APM BIOS version 1.1 only)
    #		0xff : unknown
    #        battery_status
    #		0x00 : high
    #		0x01 : low
    #		0x02 : critical
    #		0x03 : charging
    #		0xff : unknown
    #        battery_flag
    #		bit 0 : high
    #		bit 1 : low
    #		bit 2 : critical
    #		bit 3 : charging
    #		bit 7 : No system battery
    #		0xff : unknown
    #        battery_life (in percentage)
    #		0-100 : valid
    #		-1    : unknown
    #        time_units
    #		number of minutes or seconds
    #		-1 : unknown
    #        units
    #		min : minutes
    #		sec : seconds
    #
   
    if {[gets $fileId line] < 0} {
        puts "Error reading /proc/apm"
	exit
    }
    close $fileId
       
    if {[scan $line "%s %d.%d 0x%02x 0x%02x 0x%02x 0x%02x %d%% %d %s" \
    	apm(linux_driver_version) \
	apm(apm_bios_version_high) \
	apm(apm_bios_version_low) \
	apm(apm_bios_info_flags) \
	apm(ac_line_status) \
	apm(battery_status) \
	apm(battery_flag) \
	apm(percentage) \
	apm(time_units) \
	apm(units)] != 10} {

	puts "Format error on contents of /proc/apm"
	exit
    }

    #
    # Change the AC-Line image color
    #	red   : offline
    #	green : online
    #   orange: charging
    #

    switch $apm(ac_line_status) {
       0 {
           acline_xbm configure -foreground red 
	   balloon_setup .frame.acline "Offline"
       }
       1 {
           acline_xbm configure -foreground green
	   balloon_setup .frame.acline "Online"
       }
       3 {
           acline_xbm configure -foreground orange
	   balloon_setup .frame.acline "Backup Power"
       }
    }

    #
    # Change the battery image color
    #	green : high
    #   yellow: low
    #   red   : critical
    #	orange: charging
    #   
    # 	no-battery image is used to indicate no system battery.
    #

    set remain     "($apm(time_units) $apm(units))"
    set percentage "$apm(percentage)%"

    if {($apm(battery_flag) & 0x07) == 0x07} {
	.frame.battery configure -image no_battery_xbm
	balloon_setup .frame.battery "No battery"
    } else {
        switch $apm(battery_status) {
            0 {
		set options(alarm_started) 0
	        .frame.battery configure -image battery_xbm
	        battery_xbm configure -foreground green
	        balloon_setup .frame.battery "High $remain"
	    }
	    1 {
		set options(alarm_started) 0
	        .frame.battery configure -image battery_xbm
	        battery_xbm configure -foreground yellow
	        balloon_setup .frame.battery "Low $remain"
	    }
	    2 {
	        apm_alarm
	        .frame.battery configure -image battery_xbm
	        battery_xbm configure -foreground red
	        balloon_setup .frame.battery "Critical $remain"
	    }
	    3 {
		set options(alarm_started) 0
	        .frame.battery configure -image battery_xbm
	        battery_xbm configure -foreground orange
	        balloon_setup .frame.battery "Charging $percentage $remain"
	    }
        }
    }

    after [expr $options(interval) * 1000] apm_update
}

#========================================
#
# check if /proc/apm is accessable
#

if { [file exists /proc/apm] != 1 } {
    puts "File /proc/apm not found. Is kernel compiled with apm enabled ?"
    exit
}

#========================================
#
# Create the GUI
#
# Bind the options window to any button on the bitmap labels.
#

wm title . "APM check"
wm overrideredirect . 1
wm geometry . +0-1 

set w .frame
frame $w -background black
    label $w.battery -image battery_xbm
    label $w.acline -image acline_xbm
    pack $w.battery $w.acline -side left
pack .frame

bind .frame.battery <Any-ButtonPress> "apm_options"
bind .frame.acline  <Any-ButtonPress> "apm_options"

apm_update

apm_alarm
